package org.nuiton.topia;

import java.lang.reflect.Array;
import java.util.Arrays;
import org.nuiton.topia.framework.TopiaContextImplementor;
import org.nuiton.topia.persistence.TopiaDAO;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.TopiaEntityEnum;
import org.nuiton.topia.persistence.util.EntityOperator;
import org.nuiton.topia.persistence.util.EntityOperatorStore;
import org.nuiton.topia.test.entities.Person;
import org.nuiton.topia.test.entities.PersonDAO;
import org.nuiton.topia.test.entities.Pet;
import org.nuiton.topia.test.entities.PetDAO;
import org.nuiton.topia.test.entities.Race;
import org.nuiton.topia.test.entities.RaceDAO;
import org.nuiton.topiatest.Address;
import org.nuiton.topiatest.AddressDAO;
import org.nuiton.topiatest.Bill;
import org.nuiton.topiatest.BillDAO;
import org.nuiton.topiatest.Company;
import org.nuiton.topiatest.CompanyDAO;
import org.nuiton.topiatest.Department;
import org.nuiton.topiatest.DepartmentDAO;
import org.nuiton.topiatest.Employe;
import org.nuiton.topiatest.EmployeDAO;
import org.nuiton.topiatest.ExtraDAOEntity;
import org.nuiton.topiatest.ExtraDAOEntityDAO;
import org.nuiton.topiatest.NaturalizedEntity;
import org.nuiton.topiatest.NaturalizedEntityDAO;
import org.nuiton.topiatest.Personne;
import org.nuiton.topiatest.PersonneDAO;
import org.nuiton.topiatest.Product;
import org.nuiton.topiatest.ProductDAO;
import org.nuiton.topiatest.QueriedEntity;
import org.nuiton.topiatest.QueriedEntityDAO;
import org.nuiton.topiatest.Store;
import org.nuiton.topiatest.StoreDAO;
import org.nuiton.topiatest.Type;
import org.nuiton.topiatest.TypeDAO;
import org.nuiton.topiatest.deletetest.Contact2;
import org.nuiton.topiatest.deletetest.Contact2DAO;
import org.nuiton.topiatest.deletetest.Party2;
import org.nuiton.topiatest.deletetest.Party2DAO;
import org.nuiton.topiatest.deletetest.Telephone2;
import org.nuiton.topiatest.deletetest.Telephone2DAO;

public  class TopiaTestDAOHelper {



    public enum TopiaTestEntityEnum implements TopiaEntityEnum {

        Person(Person.class),
        Pet(Pet.class),
        Race(Race.class),
        Address(Address.class),
        Bill(Bill.class),
        Company(Company.class),
        Department(Department.class),
        Employe(Employe.class),
        ExtraDAOEntity(ExtraDAOEntity.class),
        NaturalizedEntity(NaturalizedEntity.class, "naturalIdNotNull", "naturalIdNull"),
        Personne(Personne.class),
        Product(Product.class),
        QueriedEntity(QueriedEntity.class),
        Store(Store.class),
        Type(Type.class),
        Contact2(Contact2.class),
        Party2(Party2.class),
        Telephone2(Telephone2.class);
        /**
         * the contract of the entity
         */
        protected Class<? extends TopiaEntity> contract;
        /**
         * the fully qualified name of the implementation of the entity
         */
        protected String implementationFQN;
        /**
         * the implementation class of the entity (will be lazy computed at runtime)
         */
        protected Class<? extends TopiaEntity> implementation;
        /**
         * the array of property involved in the natural key of the entity.
         */
        protected String[] naturalIds;

         /**
          * TopiaTestEntityEnum :
              * @param contract 
          * @param naturalIds 
          */
         TopiaTestEntityEnum(Class<? extends TopiaEntity > contract, String... naturalIds)     {
    this.contract = contract;
    this.naturalIds = naturalIds;
    this.implementationFQN = contract.getName()+"Impl";
    }

         /**
          * getContract :
              * @return Class<? extends TopiaEntity> 
          */

    @Override        public Class<? extends TopiaEntity> getContract()     {
    return contract;
    }

         /**
          * getNaturalIds :
              * @return String[] 
          */

    @Override        public String[] getNaturalIds()     {
    return naturalIds;
    }

         /**
          * getImplementationFQN :
              * @return String 
          */

    @Override        public String getImplementationFQN()     {
    return implementationFQN;
    }

         /**
          * setImplementationFQN :
              * @param implementationFQN 
          */

    @Override        public void setImplementationFQN(String implementationFQN)     {
    this.implementationFQN = implementationFQN;
    this.implementation = null;
    // on reinitialise le magasin d'operators
   EntityOperatorStore.clear();
    }

         /**
          * accept :
              * @param klass 
          * @return boolean 
          */

    @Override        public boolean accept(Class<? extends TopiaEntity> klass)     {
     return TopiaTestDAOHelper.getContractClass(klass) == contract;
    }

         /**
          * getImplementation :
              * @return Class<? extends TopiaEntity> 
          */

    @Override        public Class<? extends TopiaEntity> getImplementation()     {
 if (implementation == null) {
    try {
            implementation = (Class<? extends TopiaEntity>) Class.forName(implementationFQN);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException("could not find class " + implementationFQN);
        }
 }
 return implementation;
    }

         /**
          * valueOf :
              * @param entity 
          * @return TopiaTestEntityEnum 
          */
        public static TopiaTestEntityEnum valueOf(TopiaEntity entity)     {
     return valueOf(entity.getClass());
     }

         /**
          * valueOf :
              * @param klass 
          * @return TopiaTestEntityEnum 
          */
        public static TopiaTestEntityEnum valueOf(Class<?> klass)     {
     if (klass.isInterface()) {
        return TopiaTestEntityEnum.valueOf(klass.getSimpleName());
     }
     for (TopiaTestEntityEnum entityEnum : TopiaTestEntityEnum.values()) {
         if (entityEnum.getContract().isAssignableFrom(klass)) {
             //todo check it works for inheritance
             return entityEnum;
         }
     }
     throw new IllegalArgumentException("no entity defined for the class " + klass + " in : " + Arrays.toString(TopiaTestEntityEnum.values()));
     }


    } //TopiaTestEntityEnum

         /**
          * TopiaTestDAOHelper :
              */
        protected TopiaTestDAOHelper()     {     }

         /**
          * getModelVersion :
              * @return String 
          */
        public static String getModelVersion()     {
     return "1.2";
     }

         /**
          * getModelName :
              * @return String 
          */
        public static String getModelName()     {
     return "TopiaTest";
     }

         /**
          * getPersonDAO :
              * @param context 
          * @return PersonDAO 
          * @throws TopiaException
          */
        public static PersonDAO getPersonDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     PersonDAO result = ci.getDAO(Person.class, PersonDAO.class);
     return result;
     }

         /**
          * getPetDAO :
              * @param context 
          * @return PetDAO 
          * @throws TopiaException
          */
        public static PetDAO getPetDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     PetDAO result = ci.getDAO(Pet.class, PetDAO.class);
     return result;
     }

         /**
          * getRaceDAO :
              * @param context 
          * @return RaceDAO 
          * @throws TopiaException
          */
        public static RaceDAO getRaceDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     RaceDAO result = ci.getDAO(Race.class, RaceDAO.class);
     return result;
     }

         /**
          * getAddressDAO :
              * @param context 
          * @return AddressDAO 
          * @throws TopiaException
          */
        public static AddressDAO getAddressDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     AddressDAO result = ci.getDAO(Address.class, AddressDAO.class);
     return result;
     }

         /**
          * getBillDAO :
              * @param context 
          * @return BillDAO 
          * @throws TopiaException
          */
        public static BillDAO getBillDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     BillDAO result = ci.getDAO(Bill.class, BillDAO.class);
     return result;
     }

         /**
          * getCompanyDAO :
              * @param context 
          * @return CompanyDAO 
          * @throws TopiaException
          */
        public static CompanyDAO getCompanyDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     CompanyDAO result = ci.getDAO(Company.class, CompanyDAO.class);
     return result;
     }

         /**
          * getDepartmentDAO :
              * @param context 
          * @return DepartmentDAO 
          * @throws TopiaException
          */
        public static DepartmentDAO getDepartmentDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     DepartmentDAO result = ci.getDAO(Department.class, DepartmentDAO.class);
     return result;
     }

         /**
          * getEmployeDAO :
              * @param context 
          * @return EmployeDAO 
          * @throws TopiaException
          */
        public static EmployeDAO getEmployeDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     EmployeDAO result = ci.getDAO(Employe.class, EmployeDAO.class);
     return result;
     }

         /**
          * getExtraDAOEntityDAO :
              * @param context 
          * @return ExtraDAOEntityDAO 
          * @throws TopiaException
          */
        public static ExtraDAOEntityDAO getExtraDAOEntityDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     ExtraDAOEntityDAO result = ci.getDAO(ExtraDAOEntity.class, ExtraDAOEntityDAO.class);
     return result;
     }

         /**
          * getNaturalizedEntityDAO :
              * @param context 
          * @return NaturalizedEntityDAO 
          * @throws TopiaException
          */
        public static NaturalizedEntityDAO getNaturalizedEntityDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     NaturalizedEntityDAO result = ci.getDAO(NaturalizedEntity.class, NaturalizedEntityDAO.class);
     return result;
     }

         /**
          * getPersonneDAO :
              * @param context 
          * @return PersonneDAO 
          * @throws TopiaException
          */
        public static PersonneDAO getPersonneDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     PersonneDAO result = ci.getDAO(Personne.class, PersonneDAO.class);
     return result;
     }

         /**
          * getProductDAO :
              * @param context 
          * @return ProductDAO 
          * @throws TopiaException
          */
        public static ProductDAO getProductDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     ProductDAO result = ci.getDAO(Product.class, ProductDAO.class);
     return result;
     }

         /**
          * getQueriedEntityDAO :
              * @param context 
          * @return QueriedEntityDAO 
          * @throws TopiaException
          */
        public static QueriedEntityDAO getQueriedEntityDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     QueriedEntityDAO result = ci.getDAO(QueriedEntity.class, QueriedEntityDAO.class);
     return result;
     }

         /**
          * getStoreDAO :
              * @param context 
          * @return StoreDAO 
          * @throws TopiaException
          */
        public static StoreDAO getStoreDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     StoreDAO result = ci.getDAO(Store.class, StoreDAO.class);
     return result;
     }

         /**
          * getTypeDAO :
              * @param context 
          * @return TypeDAO 
          * @throws TopiaException
          */
        public static TypeDAO getTypeDAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     TypeDAO result = ci.getDAO(Type.class, TypeDAO.class);
     return result;
     }

         /**
          * getContact2DAO :
              * @param context 
          * @return Contact2DAO 
          * @throws TopiaException
          */
        public static Contact2DAO getContact2DAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     Contact2DAO result = ci.getDAO(Contact2.class, Contact2DAO.class);
     return result;
     }

         /**
          * getParty2DAO :
              * @param context 
          * @return Party2DAO 
          * @throws TopiaException
          */
        public static Party2DAO getParty2DAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     Party2DAO result = ci.getDAO(Party2.class, Party2DAO.class);
     return result;
     }

         /**
          * getTelephone2DAO :
              * @param context 
          * @return Telephone2DAO 
          * @throws TopiaException
          */
        public static Telephone2DAO getTelephone2DAO(TopiaContext context) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     Telephone2DAO result = ci.getDAO(Telephone2.class, Telephone2DAO.class);
     return result;
     }

         /**
          * getDAO :
              * @param context 
          * @param klass 
          * @return <T extends TopiaEntity, D extends TopiaDAO<? super T>> D 
          * @throws TopiaException
          */
        public static <T extends TopiaEntity, D extends TopiaDAO<? super T>> D getDAO(TopiaContext context, Class<T> klass) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     TopiaTestEntityEnum constant = TopiaTestEntityEnum.valueOf(klass);
     D dao = (D) ci.getDAO(constant.getContract());
     return dao;
     }

         /**
          * getDAO :
              * @param context 
          * @param entity 
          * @return <T extends TopiaEntity, D extends TopiaDAO<? super T>> D 
          * @throws TopiaException
          */
        public static <T extends TopiaEntity, D extends TopiaDAO<? super T>> D getDAO(TopiaContext context, T entity) throws TopiaException     {
     TopiaContextImplementor ci = (TopiaContextImplementor) context;
     TopiaTestEntityEnum constant = TopiaTestEntityEnum.valueOf(entity);
     D dao = (D) ci.getDAO(constant.getContract());
     return dao;
     }

         /**
          * getContractClass :
              * @param klass 
          * @return <T extends TopiaEntity> Class<T> 
          */
        public static <T extends TopiaEntity> Class<T> getContractClass(Class<T> klass)     {
     TopiaTestEntityEnum constant = TopiaTestEntityEnum.valueOf(klass);
     return (Class<T>) constant.getContract();
     }

         /**
          * getImplementationClass :
              * @param klass 
          * @return <T extends TopiaEntity> Class<T> 
          */
        public static <T extends TopiaEntity> Class<T> getImplementationClass(Class<T> klass)     {
     TopiaTestEntityEnum constant = TopiaTestEntityEnum.valueOf(klass);
     return (Class<T>) constant.getImplementation();
     }

         /**
          * getContractClasses :
              * @return Class<? extends TopiaEntity>[] 
          */
        public static Class<? extends TopiaEntity>[] getContractClasses()     {
     TopiaTestEntityEnum[] values = TopiaTestEntityEnum.values();
     Class<? extends TopiaEntity>[] result = (Class<? extends TopiaEntity>[]) Array.newInstance(Class.class, values.length);
     for (int i = 0; i < values.length; i++) {
         result[i] = values[i].getContract();
     }
     return result;
     }

         /**
          * getImplementationClasses :
              * @return Class<? extends TopiaEntity>[] 
          */
        public static Class<? extends TopiaEntity>[] getImplementationClasses()     {
     TopiaTestEntityEnum[] values = TopiaTestEntityEnum.values();
     Class<? extends TopiaEntity>[] result = (Class<? extends TopiaEntity>[]) Array.newInstance(Class.class, values.length);
     for (int i = 0; i < values.length; i++) {
         result[i] = values[i].getImplementation();
     }
     return result;
     }

         /**
          * getImplementationClassesAsString :
              * @return String 
          */
        public static String getImplementationClassesAsString()     {
     StringBuilder buffer = new StringBuilder();
     for (Class<? extends TopiaEntity> aClass : getImplementationClasses()) {
         buffer.append(',').append(aClass.getName());
     }
     return buffer.substring(1);
     }

         /**
          * getContracts :
              * @return TopiaTestEntityEnum[] 
          */
        public static TopiaTestEntityEnum[] getContracts()     {
     return TopiaTestEntityEnum.values();
     }

         /**
          * getOperator :
              * @param klass 
          * @return <T extends TopiaEntity> EntityOperator<T> 
          */
        public static <T extends TopiaEntity> EntityOperator<T> getOperator(Class<T> klass)     {
     TopiaTestEntityEnum constant = TopiaTestEntityEnum.valueOf(klass);
     return (EntityOperator<T>) EntityOperatorStore.getOperator(constant);
     }


    } //TopiaTestDAOHelper
