package org.nuiton.topia.test.entities;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.framework.TopiaContextImplementor;
import org.nuiton.topia.persistence.TopiaDAO;
import org.nuiton.topia.persistence.TopiaDAOImpl;
import org.nuiton.topia.persistence.TopiaEntity;

public  abstract class PersonDAOAbstract<E extends Person> extends TopiaDAOImpl<E> implements TopiaDAO<E> {


     /**
      * getEntityClass :
      * @return Class<E> 
      */
    public Class<E> getEntityClass() {
        return (Class<E>)Person.class;
    }

     /**
      * delete :
      * @param entity 
      * @throws TopiaException
      */
    public void delete(E entity) throws TopiaException {
                                        {
                                        List<Pet> list = getContext()
                                                    .getDAO(Pet.class)
                                                    .findAllByProperties(Pet.PROPERTY_PERSON, entity);
                                            for (Pet item : list) {

                                                // sletellier : Set null only if target is concerned by deletion
                                                if (entity.equals(item.getPerson())) {
                                                    item.setPerson(null);
                                                }
                        
                    }
                }

        super.delete(entity);
    }

     /**
      * findByName :
      * Retourne le premier élément trouvé ayant comme valeur pour l'attribut name le paramètre.
      * @param v 
      * @return E 
      * @throws TopiaException
      */
    public E findByName(String v) throws TopiaException {
        E result = findByProperty(Person.PROPERTY_NAME, v);
        return result;
    }

     /**
      * findAllByName :
      * Retourne les éléments ayant comme valeur pour l'attribut name le paramètre.
      * @param v 
      * @return List<E> 
      * @throws TopiaException
      */
    public List<E> findAllByName(String v) throws TopiaException {
        List<E> result = findAllByProperty(Person.PROPERTY_NAME, v);
        return result;
    }

     /**
      * findByFirstname :
      * Retourne le premier élément trouvé ayant comme valeur pour l'attribut firstname le paramètre.
      * @param v 
      * @return E 
      * @throws TopiaException
      */
    public E findByFirstname(String v) throws TopiaException {
        E result = findByProperty(Person.PROPERTY_FIRSTNAME, v);
        return result;
    }

     /**
      * findAllByFirstname :
      * Retourne les éléments ayant comme valeur pour l'attribut firstname le paramètre.
      * @param v 
      * @return List<E> 
      * @throws TopiaException
      */
    public List<E> findAllByFirstname(String v) throws TopiaException {
        List<E> result = findAllByProperty(Person.PROPERTY_FIRSTNAME, v);
        return result;
    }

     /**
      * findContainsPet :
      * Retourne le premier élément ayant comme valeur pour l'attribut pet le paramètre.
      * @param v 
      * @return E 
      * @throws TopiaException
      */
    public E findContainsPet(Pet v) throws TopiaException {
        E result = findContains(Person.PROPERTY_PET, v);
        return result;
    }

     /**
      * findAllContainsPet :
      * Retourne les éléments ayant comme valeur pour l'attribut pet le paramètre.
      * @param v 
      * @return List<E> 
      * @throws TopiaException
      */
    public List<E> findAllContainsPet(Pet v) throws TopiaException {
        List<E> result = findAllContains(Person.PROPERTY_PET, v);
        return result;
    }

     /**
      * findUsages :
      * @param type 
      * @param entity 
      * @return <U extends TopiaEntity> List<U> 
      * @throws TopiaException
      */

    @Override    public <U extends TopiaEntity> List<U> findUsages(Class<U> type, E entity) throws TopiaException {
        List<?> result = new ArrayList();
        List tmp;

        if (type == Pet.class) {
            PetDAO dao = (PetDAO)
                getContext().getDAO(Pet.class);
            tmp = dao.findAllByPerson(entity);
            result.addAll(tmp);
        }

        return (List<U>) result;
    }

     /**
      * findAllUsages :
      * @param entity 
      * @return Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> 
      * @throws TopiaException
      */

    @Override    public Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> findAllUsages(E entity) throws TopiaException {
        Map<Class<? extends TopiaEntity>,List<? extends TopiaEntity>> result;
        result = new HashMap<Class<? extends TopiaEntity>, List<? extends TopiaEntity>>(1);

        List<? extends TopiaEntity> list;

        list = findUsages(Pet.class, entity);
        if (!list.isEmpty()) {
            result.put(Pet.class, list);
        }

        return result;
    }


} //PersonDAOAbstract<E extends Person>
