package org.nuiton.topia.test.entities;

import java.util.ArrayList;
import java.util.List;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.nuiton.i18n.I18n;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.persistence.EntityVisitor;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.TopiaEntityAbstract;

public  abstract class PetAbstract extends TopiaEntityAbstract implements Pet {

    /**
     * Nom de l'attribut en BD : name

     */
    protected String name;
    /**
     * Nom de l'attribut en BD : type

     */
    protected String type;
    /**
     * Nom de l'attribut en BD : person

     */
    protected Person person;
    /**
     * Nom de l'attribut en BD : race

     */
    protected Race race;
    private static final long serialVersionUID = 7075496071026585907L;

static {
    
        I18n.n_("topia.test.common.pet");
        I18n.n_("topia.test.common.name");
        I18n.n_("topia.test.common.type");
        I18n.n_("topia.test.common.person");
        I18n.n_("topia.test.common.race");
    }
     /**
      * accept :
      * Envoi via les methodes du visitor l'ensemble des champs de l'entity
avec leur nom, type et valeur.
      * @param visitor le visiteur de l'entite.
      * @throws TopiaException
      */
    public void accept(EntityVisitor visitor) throws TopiaException {
        visitor.start(this);
        visitor.visit(this, PROPERTY_NAME, String.class, name);
        visitor.visit(this, PROPERTY_TYPE, String.class, type);
        visitor.visit(this, PROPERTY_PERSON, Person.class, person);
        visitor.visit(this, PROPERTY_RACE, Race.class, race);
        visitor.end(this);
}

     /**
      * setName :
      * @param name 
      */

    @Override    public void setName(String name) {
        String oldValue = this.name;
        fireOnPreWrite(PROPERTY_NAME, oldValue, name);
        this.name = name;
        fireOnPostWrite(PROPERTY_NAME, oldValue, name);
}

     /**
      * getName :
      * @return String 
      */

    @Override    public String getName() {
        fireOnPreRead(PROPERTY_NAME, name);
        String result = this.name;
        fireOnPostRead(PROPERTY_NAME, name);
        return result;
}

     /**
      * setType :
      * @param type 
      */

    @Override    public void setType(String type) {
        String oldValue = this.type;
        fireOnPreWrite(PROPERTY_TYPE, oldValue, type);
        this.type = type;
        fireOnPostWrite(PROPERTY_TYPE, oldValue, type);
}

     /**
      * getType :
      * @return String 
      */

    @Override    public String getType() {
        fireOnPreRead(PROPERTY_TYPE, type);
        String result = this.type;
        fireOnPostRead(PROPERTY_TYPE, type);
        return result;
}

     /**
      * setPerson :
      * @param person 
      */

    @Override    public void setPerson(Person person) {
        Person oldValue = this.person;
        fireOnPreWrite(PROPERTY_PERSON, oldValue, person);
        this.person = person;
        fireOnPostWrite(PROPERTY_PERSON, oldValue, person);
}

     /**
      * getPerson :
      * @return Person 
      */

    @Override    public Person getPerson() {
        fireOnPreRead(PROPERTY_PERSON, person);
        Person result = this.person;
        fireOnPostRead(PROPERTY_PERSON, person);
        return result;
}

     /**
      * setRace :
      * @param race 
      */

    @Override    public void setRace(Race race) {
        Race oldValue = this.race;
        fireOnPreWrite(PROPERTY_RACE, oldValue, race);
        this.race = race;
        fireOnPostWrite(PROPERTY_RACE, oldValue, race);
}

     /**
      * getRace :
      * @return Race 
      */

    @Override    public Race getRace() {
        fireOnPreRead(PROPERTY_RACE, race);
        Race result = this.race;
        fireOnPostRead(PROPERTY_RACE, race);
        return result;
}

     /**
      * getAggregate :
      * @return List<TopiaEntity> 
      * @throws TopiaException
      */

    @Override    public List<TopiaEntity> getAggregate() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            result.add(entity);
            result.addAll(entity.getAggregate());
        }

        return result;
}

     /**
      * getComposite :
      * @return List<TopiaEntity> 
      * @throws TopiaException
      */

    @Override    public List<TopiaEntity> getComposite() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp
        tmp.add(getRace());

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            if (entity != null) {
                result.add(entity);
                result.addAll(entity.getComposite());
            }
        }

        return result;
    }

     /**
      * toString :
      * @return String 
      */

    @Override    public String toString() {
        String result = new ToStringBuilder(this).
            append(PROPERTY_NAME, this.name).
            append(PROPERTY_TYPE, this.type).
            append(PROPERTY_RACE, this.race).
         toString();
        return result;
}


} //PetAbstract
