package org.nuiton.topiatest;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.framework.TopiaContextImplementor;
import org.nuiton.topia.persistence.TopiaDAO;
import org.nuiton.topia.persistence.TopiaDAOImpl;
import org.nuiton.topia.persistence.TopiaEntity;

public  abstract class ProductDAOAbstract<E extends Product> extends TopiaDAOImpl<E> implements TopiaDAO<E> {


     /**
      * getEntityClass :
      * @return Class<E> 
      */
    public Class<E> getEntityClass() {
        return (Class<E>)Product.class;
    }

     /**
      * delete :
      * @param entity 
      * @throws TopiaException
      */
    public void delete(E entity) throws TopiaException {
        super.delete(entity);
    }

     /**
      * findByName :
      * Retourne le premier élément trouvé ayant comme valeur pour l'attribut name le paramètre.
      * @param v 
      * @return E 
      * @throws TopiaException
      */
    public E findByName(String v) throws TopiaException {
        E result = findByProperty(Product.PROPERTY_NAME, v);
        return result;
    }

     /**
      * findAllByName :
      * Retourne les éléments ayant comme valeur pour l'attribut name le paramètre.
      * @param v 
      * @return List<E> 
      * @throws TopiaException
      */
    public List<E> findAllByName(String v) throws TopiaException {
        List<E> result = findAllByProperty(Product.PROPERTY_NAME, v);
        return result;
    }

     /**
      * findByType :
      * Retourne le premier élément trouvé ayant comme valeur pour l'attribut type le paramètre.
      * @param v 
      * @return E 
      * @throws TopiaException
      */
    public E findByType(Type v) throws TopiaException {
        E result = findByProperty(Product.PROPERTY_TYPE, v);
        return result;
    }

     /**
      * findAllByType :
      * Retourne les éléments ayant comme valeur pour l'attribut type le paramètre.
      * @param v 
      * @return List<E> 
      * @throws TopiaException
      */
    public List<E> findAllByType(Type v) throws TopiaException {
        List<E> result = findAllByProperty(Product.PROPERTY_TYPE, v);
        return result;
    }

     /**
      * findUsages :
      * @param type 
      * @param entity 
      * @return <U extends TopiaEntity> List<U> 
      * @throws TopiaException
      */

    @Override    public <U extends TopiaEntity> List<U> findUsages(Class<U> type, E entity) throws TopiaException {
        List<?> result = new ArrayList();
        List tmp;

        if (type == Department.class) {
            DepartmentDAO dao = (DepartmentDAO)
                getContext().getDAO(Department.class);
            tmp = dao.findAllContainsProduct(entity);
            result.addAll(tmp);
        }

        return (List<U>) result;
    }

     /**
      * findAllUsages :
      * @param entity 
      * @return Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> 
      * @throws TopiaException
      */

    @Override    public Map<Class<? extends TopiaEntity>, List<? extends TopiaEntity>> findAllUsages(E entity) throws TopiaException {
        Map<Class<? extends TopiaEntity>,List<? extends TopiaEntity>> result;
        result = new HashMap<Class<? extends TopiaEntity>, List<? extends TopiaEntity>>(1);

        List<? extends TopiaEntity> list;

        list = findUsages(Department.class, entity);
        if (!list.isEmpty()) {
            result.put(Department.class, list);
        }

        return result;
    }


} //ProductDAOAbstract<E extends Product>
